/***************************************************************************
                       vlconfigitem.h -  description
                            -------------------
    begin                : Mon Jul 22 2002
    copyright            : (C) 2002 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLCONFIGITEM_H_
#define _VLCONFIGITEM_H_


#include "vldef.h"

#include <string>
#include <vector>
#include <map>
#include <vigra/stdimage.hxx>

namespace vistaLab {

class module;
class configurableObject;
class slice;


////////////////////////// Misc Functions/Classes ///////////////////////////


/***************************************************************************/
//! Helper class which gets stored in CVistaModule for each configurable item.
/*!
 *
 *  It has a type and stores the information which gets passed by the
 *  addConfig... - functions
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class VISTALABBASEEXPORT configItem {
public:

    typedef const char *(configurableObject::*TValidatorFunc)();
    typedef void (module::*TRenderFunc)(vigra::BRGBImage *);
    typedef void (configurableObject::*TMemberFunc)();

    typedef bool (module::*TUnitTest)();

    typedef const std::string (module::*TLogFunc)();

    // Enumerates all possible configuration types
    enum EType{
        INVALID,
        STRING,
        FILENAME,
        INT,
        VALIDATORFUNC,
        MULTIPLEFILENAMES,
        BOOL,
        ACTIVEBOOL, // Special bool for the active - flag: It does not get shown in a dialog
        DOUBLE,
        SINGLECHOICE,
        MULTIPLECHOICE,
        CUSTOMCHOICE,
        INPUTPIN,
        OUTPUTPIN,
        RENDERINPUTPIN,
        RENDEROUTPUTPIN,
        LOGINPUTPIN,
        LOGOUTPUTPIN,
        UNITTESTPIN,
        SAVEFILENAME
    };


    /***************************************************************************/
    /*!
     *  Takes a void target pointer
     *
     ***************************************************************************/
    configItem(const EType type,
        const void *target,
        const std::string &description,
        const std::vector< std::string > &selections
        = std::vector< std::string >(0));


    /***************************************************************************/
    /*!
     *  Takes a validator function pointer
     *
     ***************************************************************************/
    configItem(const EType type,
        const TMemberFunc targetFunc,
        const std::string &description,
        const std::vector< std::string > &selections
        = std::vector< std::string >(0));

    EType                    m_type;
    union{
        const void           *data;
        TMemberFunc           func;
    } m_target;

    std::string              m_description;
    std::string              m_localizedDescription;
    std::vector<std::string> m_selections;
};

} // namespace vistaLab

#endif // #ifndef _VLCONFIGITEM_H_
