/***************************************************************************
                        vlglwindow.h  -  description
                            -------------------
    begin                : Mon Jul 07 2003
    copyright            : (C) 2003 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLGLWINDOW_H_
#define _VLGLWINDOW_H_

#include <wx/glcanvas.h>
#include <wx/dialog.h>
#include <wx/timer.h>

#include "vldef.h"

#include <vector>

namespace vistaLab {

/***************************************************************************/
//! Client window which displays OpenGL Data
/*!
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class VISTALABBASEEXPORT GLWindow : public wxGLCanvas{
public:
    GLWindow(wxWindow *parent, int id,
        const wxPoint  &position = wxDefaultPosition,
        const wxSize   &size     = wxDefaultSize);
    ~GLWindow();
    void done();
    void show(bool do_show=true);
    void setTitle(const wxString &title);

protected:
    wxCriticalSection m_openGLLock;
    virtual void draw();
    virtual void initializeGL();
    virtual void initializeCamera();
    virtual void placeCamera();
    void setPlanes(float newnear, float newfar);
    void setObjectSize(float size);
    GLdouble m_zoom;

private:
    // Helper functions for OpenGL processing
    bool m_initialized;
    bool m_drawBackground;
    bool m_transparency;
    bool m_optimization;
    bool m_suppressText;
    bool m_kill;
    wxPoint m_rightClickPos;

    // Callbacks for user interaction / refresh - calls from the framework
    void onPaint             (wxPaintEvent   &event);
    void onMouse             (wxMouseEvent   &event);
    void onSize              (wxSizeEvent    &event);
    void onSnapshot          (wxCommandEvent &event);
    void onToggleBackground  (wxCommandEvent &event);
    void onToggleTransparency(wxCommandEvent &event);
    void onToggleOptimization(wxCommandEvent &event);
    void onToggleSuppressText(wxCommandEvent &event);
    void onReadQuats         (wxCommandEvent &event);
    void onWriteQuats        (wxCommandEvent &event);
    void onEraseBackground   (wxEraseEvent   &event);
    void onTimer             (wxTimerEvent   &event);

    // Members for the viewpoint
    float m_trackballRot[4];
    GLdouble m_trans[3];

    // The "tracked" mouse position
    wxPoint  m_mousePos;
    double   m_mouseSpeed[2];
    float    m_near;
    float    m_far;

    int      m_wheelRot;
    float    m_backTrans;
    wxTimer *m_spinTimer;
    wxMouseEvent m_mouseEvent;

    DECLARE_EVENT_TABLE()
};

template< class Widget > Widget *GLwin(const wxString &title,
                                               const wxSize &size,
                                               bool resize = true)
{
    wxDialog *dialog = new wxDialog((wxWindow *) NULL,-1,title,wxDefaultPosition,size,wxDEFAULT_DIALOG_STYLE |
                                                                 (resize ? wxRESIZE_BORDER : 0 ) );
    Widget *widget = new Widget(dialog,-1,wxDefaultPosition,size);
    widget->SetSize(size);
    dialog->Centre();
    return widget;
}

} // namespace vistaLab

#endif // #ifndef _VLGLWINDOW_H_
