/***************************************************************************
                         vlslice.h  -  description
                            -------------------
    begin                : Fri Nov 16 2001
    copyright            : (C) 2001 by Fabian Wenzel
    email                : wenzel@tu-harburg.de

***************************************************************************/

/***************************************************************************
 *                                                                         *
 * This file is released under the the terms of the VistaLab public        *
 * License V1.5. See file COPYING for details.                             *
 *                                                                         *
 ***************************************************************************/

#ifndef _VLSLICE_H_
#define _VLSLICE_H_

#include <vector>

namespace vistaLab {

/***************************************************************************/
//! Encapsulates a vector containing all data items from all modules
/*!
 *  Image processing may require two or more images (e.g. for correspondence
 *  search etc.) This is why the images of different input channels are
 *  stored in one slice. Also, each module may store data when processing
 *  images, so these items get also stored here
 *
 * \ingroup Datatypes
 *
 * \author  Fabian Wenzel
 *
 ***************************************************************************/
class slice : public std::vector< void *>{
    // Document may access the validEntries - counter
public:


    /***************************************************************************/
    /*!
     *  Constructor initializes the slice with number of channels and sets
     *  all frame pointers to zero.
     *
     ***************************************************************************/
    slice();

    ~slice();


    /***************************************************************************/
    /*!
     *  Checks if there is already data at positions where the pending slice
     *  has entries
     *
     ***************************************************************************/
    bool slice::containsData(const slice &pendingData)
    {
        std::vector< void *>::const_iterator currentPendingSlot = pendingData.begin();
        std::vector< void *>::const_iterator currentTargetSlot;

        for(currentTargetSlot  = begin();
            currentTargetSlot != end();
            ++currentTargetSlot,++currentPendingSlot)
            if( (*currentPendingSlot) && (*currentTargetSlot) )
                return true;

        return false;
    }

    /***************************************************************************/
    /*!
     *  Checks if there is already data at positions where the pending slice
     *  has entries
     *
     ***************************************************************************/
    void slice::merge(slice &pendingData)
    {
        std::vector< void *>::iterator currentPendingSlot = pendingData.begin();
        std::vector< void *>::iterator currentTargetSlot;
        for(currentTargetSlot  = begin();
            currentTargetSlot != end();
            ++currentTargetSlot,++currentPendingSlot)
            if( *currentPendingSlot ){
                *currentTargetSlot  = *currentPendingSlot;
                *currentPendingSlot = 0;
            }
    }

    void slice::increaseValidEntries()
    {
        ++m_validEntries;
    }

    int slice::getValidEntries()
    {
        return m_validEntries;
    }
private:
    //! Counts if all channels contain valid data
    int m_validEntries;

};

} // namespace vistaLab

#endif // #ifndef _VLSLICE_H_
